@@ -73,9 +73,10 @@ def check_or_create_dir(wdir, uuid):
 
 # List of audit files.
 # When no ballot have been cast yet, ballots.jsons does not exist.
+# We also put here a fake filename, for the hash of the voterlist.
 audit_files=['election.json', 'public_creds.txt', 'trustees.json',
         'ballots', 'index.html']
-optional_audit_files=['ballots.jsons','result.json','shuffles.jsons']
+optional_audit_files=['ballots.jsons','result.json','shuffles.jsons','hash_voterlist']
 
 def download_audit_data(url, uuid):
     link = url + '/elections/' + uuid
@@ -152,14 +153,37 @@ def write_and_verify_new_data(wdir, uuid, data):
 def check_hash_ballots(data):
     dom = xml.dom.minidom.parseString(data['ballots'])
     list_ballots = dom.getElementsByTagName("li")
+    if len(list_ballots) == 0:
+        return Status(False, b"")
+    if len(list_ballots[0].childNodes) == 2:
+        has_weight = True
+    else:
+        has_weight = False
+
     list_hash = [ x.firstChild.firstChild.data for x in list_ballots ]
+    if has_weight:
+        list_weights = [ x.childNodes[1].data for x in list_ballots ]
+        list_weights = [ x[2:-1] for x in list_weights ]
 
     list_hash2 = []
+    list_weights2 = []
     for l in data['ballots.jsons'].splitlines():
         m = hashlib.sha256()
         m.update(l)
         h = base64.b64encode(m.digest()).decode().strip('=')
         list_hash2.append(h)
+        if has_weight:
+            jsn = json.loads(l)
+            cred = jsn['signature']['public_key']
+            pat = re.compile(cred + r',(\d+)')
+            m=pat.search(data['public_creds.txt'].decode())
+            if m:
+                list_weights2.append(m.group(1))
+            else: # if absent, default weight is 1
+                list_weights2.append('1')
+    if has_weight:
+        list_hash = [ (list_hash[i],list_weights[i]) for i in range(len(list_hash)) ]
+        list_hash2 = [ (list_hash2[i],list_weights2[i]) for i in range(len(list_hash2)) ]
     list_hash.sort()
     list_hash2.sort()
     if (not list_hash == list_hash2):
@@ -175,6 +199,8 @@ def check_index_html(data):
     # when the election is closed, there is a "disabled" attributed
     # without value that the xml parser does not like. We remove it.
     st = data['index.html'].decode().replace('disabled>Start', '>Start')
+    # also, the &lang in the link makes the parser crazy.
+    st = st.replace('&lang', 'lang')
     dom = xml.dom.minidom.parseString(st)
     fail = False
     msg = b""
@@ -207,6 +233,59 @@ def check_index_html(data):
         fail = True
     else:
         logme("  cred fingerprint ok")
+    
+    # if weights, check the total/min/max vs content of public_creds.txt
+    node = [ x.firstChild for x in dom.getElementsByTagName("div")
+            if x.firstChild != None and
+            x.firstChild.nodeType == xml.dom.minidom.Node.TEXT_NODE and
+            re.search("The total weight is",x.firstChild.data) != None ]
+    if (len(node) == 1):
+        fail_weights = False
+        pat = re.compile(r'The total weight is (\d+) \(min: (\d+), max: (\d+)\)')
+        mat = pat.match(node[0].data)
+        w_tot = int(mat.group(1))
+        w_min = int(mat.group(2))
+        w_max = int(mat.group(3))
+        c_tot = 0
+        c_min = 10000000000
+        c_max = 0
+        for l in data['public_creds.txt'].splitlines():
+            wt = l.decode().split(',')
+            if len(wt) == 2:
+                x = int(l.decode().split(',')[1])
+            else: # if absent, default weight is 1.
+                assert len(wt) == 1;
+                x = 1
+            c_tot += x
+            if (x < c_min):
+                c_min = x
+            if (x > c_max):
+                c_max = x
+        if (c_tot != w_tot) or (c_min != w_min) or (c_max != w_max):
+            msg = msg + "Error: Wrong stats of weights in election {}\n".format(uuid).encode()
+            logme(" " + str(c_tot) + " " + str(c_min) + " " + str(c_max))
+            fail = True
+        else:
+            logme("  stats of weights ok")
+
+    # check that the printed number of voters is consistent with number
+    # of credentials
+    node = [ x.firstChild for x in dom.getElementsByTagName("div")
+            if x.firstChild != None and
+            x.firstChild.nodeType == xml.dom.minidom.Node.TEXT_NODE and
+            re.search("The voter list has",x.firstChild.data) != None ]
+    assert len(node) == 1
+    pat = re.compile(r'The voter list has (\d+) voter\(s\) and fingerprint ([\w/+]+).')
+    mat = pat.match(node[0].data)
+    nb_voters = int(mat.group(1))
+    data['hash_voterlist'] = mat.group(2).encode() # for further checks
+    nb_creds = data['public_creds.txt'].decode().count('\n')
+    if (nb_voters != nb_creds):
+        msg = msg + "Error: Number of voters different from number of credentials for election {}\n".format(uuid).encode()
+        logme(" " + str(nb_voters) + " " + str(nb_creds))
+        fail = True
+    else:
+        logme("  number of voters ok")
 
     def hash_pub_key(s):
         m = hashlib.sha256()
@@ -218,14 +297,17 @@ def check_index_html(data):
     # trustees fingerprint vs trustees.json
     jsn = json.loads(data['trustees.json'])
     names = []
-    hashs = []
+    pks = []
+    certs = []
+
     for trustee in jsn:
         if trustee[0] == 'Single':
             if 'name' in trustee[1]:
                 names.append(trustee[1]['name'])
             else:
                 names.append("N/A")
-            hashs.append(hash_pub_key(trustee[1]['public_key']))
+            pks.append(hash_pub_key(trustee[1]['public_key']))
+            certs.append(None)
         else:
             assert trustee[0] == 'Pedersen'
             for tru in trustee[1]['verification_keys']:
@@ -233,47 +315,42 @@ def check_index_html(data):
                     names.append(tru['name'])
                 else:
                     names.append("N/A")
-                hashs.append(hash_pub_key(tru['public_key']))
-    pat = re.compile('(^.*) \(([\w+/]*)\)$')
+                pks.append(hash_pub_key(tru['public_key']))
+            for tru in trustee[1]['certs']:
+                m = hashlib.sha256()
+                m.update(tru['message'].encode())
+                certs.append(base64.b64encode(m.digest()).decode().strip('='))
+
     names2 = []
-    hashs2 = []
-    # in index.html, the trustees are in the ul list with id "trustees"
-    for trustee in [ x for x in dom.getElementsByTagName("ul") if
-            x.getAttribute('id') == 'trustees'
-            ][0].getElementsByTagName("li"):
-        s = trustee.firstChild.data
-        mat = pat.match(s)
-        names2.append(mat.group(1))
-        hashs2.append(mat.group(2))
-    if ((not names == names2) or (not hashs == hashs2)):
+    pks2 = []
+    certs2 = []
+    # in index.html, the non-threshold trustees are in the ul list with id "trustees"
+    arr = [ x for x in dom.getElementsByTagName("ul") if x.getAttribute('id') == 'trustees' ]
+    if arr != []:
+        pat = re.compile(r'(^.*) \(([\w+/]*)\)$')
+        for trustee in arr[0].getElementsByTagName("li"):
+            s = trustee.firstChild.data
+            mat = pat.match(s)
+            names2.append(mat.group(1))
+            pks2.append(mat.group(2))
+            certs2.append(None)
+    # the threshold trustees are in the ul list with class "trustees_threshold"
+    arr = [ x for x in dom.getElementsByTagName("ul") if x.getAttribute('class') == 'trustees_threshold' ]
+    if arr != []:
+        pat = re.compile(r'(^.*) \(([\w+/]*)\) \[([\w+/]*)\]$')
+        for trustee in arr[0].getElementsByTagName("li"):
+            s = trustee.firstChild.data
+            mat = pat.match(s)
+            names2.append(mat.group(1))
+            pks2.append(mat.group(2))
+            certs2.append(mat.group(3))
+    
+    if ((not names == names2) or (not pks == pks2) or (not certs == certs2)):
         msg = msg + "Error: Wrong trustees fingerprint of election {}\n".format(uuid).encode()
         fail = True
     else:
         logme("  trustees fingerprints ok")
 
-    # In Pedersen mode, check PKI fingerprints
-    if jsn[0][0] == 'Pedersen':
-        hashs = []
-        for trustee in jsn[0][1]['certs']:
-            m = hashlib.sha256()
-            m.update(trustee['message'].encode())
-            hashs.append(base64.b64encode(m.digest()).decode().strip('='))
-        names2 = []
-        hashs2 = []
-        # in index.html, the PKI are in the ul list with id "pki"
-        for trustee in [ x for x in dom.getElementsByTagName("ul") if
-                x.getAttribute('id') == 'pki'
-                ][0].getElementsByTagName("li"):
-            s = trustee.firstChild.data
-            mat = pat.match(s)
-            names2.append(mat.group(1))
-            hashs2.append(mat.group(2))
-        if ((not names == names2) or (not hashs == hashs2)):
-            msg = msg + "Error: Wrong PKI fingerprint of election {}\n".format(uuid).encode()
-            fail = True
-        else:
-            logme("  PKI fingerprints ok")
-
     # encrypted tally vs result.json (if present)
     # (and extract shuffles if present)
     shuf_array = None
@@ -378,6 +455,7 @@ parser.add_argument("--wdir", required=True, help="work dir where logs are kept"
 parser.add_argument("--checkhash", type=str2bool, nargs='?',
                         const=True, default=True, metavar="yes|no",
                         help="also check static files on the server")
+parser.add_argument("--hashref", help="reference file for hash, as used by check_hash.py")
 parser.add_argument("--logfile", help="file to write the non-error logs")
 
 args = parser.parse_args()
@@ -401,6 +479,12 @@ if not os.path.isdir(args.wdir) or not os.access(args.wdir, os.W_OK | os.R_OK):
     print("The wdir {} should read/write accessible".format(args.wdir))
     sys.exit(1)
 
+# check that a ref file is given if checkhash is set
+if args.checkhash == True:
+    if not args.hashref:
+        print("If --checkhash is set, a --hashref file should be given")
+        sys.exit(1)
+
 logme("[{}] Starting monitoring elections.".format(datetime.datetime.now()))
 
 for uuid in uuids:
@@ -420,6 +504,18 @@ for uuid in uuids:
 
         stat = check_index_html(data)
         status.merge(stat)
+        # create the hash_voterlist file, with the value read from index.html
+        # or check that its value is consistent
+        p = os.path.join(args.wdir, uuid, 'hash_voterlist')
+        if os.path.exists(p):
+            with open(p, "rb") as file:
+                oldhash = file.read()
+            if (oldhash != data['hash_voterlist']):
+                status.merge(Status(True,
+                    "Error: hash of the voter list changed for election {}".format(uuid).encode()))
+        else:
+            with open(p, "wb") as file:
+                file.write(data['hash_voterlist'])
 
     # commit 
     if status.msg != b'':
@@ -431,7 +527,7 @@ for uuid in uuids:
 # we dot it only once
 
 if args.checkhash == True:
-    hh = subprocess.run(["check_hash.py", "--url", args.url ],
+    hh = subprocess.run(["check_hash.py", "--url", args.url, "--reference", args.hashref ],
             stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     if hh.returncode != 0:
         print(hh.stdout.decode())
