@@ -2,13 +2,19 @@
 
 set -e
 
-if [ -d _build ]; then
+if [ "$1" = "--preload" ]; then
+    shift
+    . "$1"
+    shift
+fi
+
+if [ -e .git ]; then
     : ${BELENIOS_CONFIG:=demo/ocsigenserver.conf.in}
     : ${BELENIOS_VARDIR:=_run}
     : ${BELENIOS_RUNDIR:=/tmp/belenios}
-    : ${BELENIOS_LIBDIR:=_build/lib}
-    : ${BELENIOS_STATICDIR:=_build/src/static}
-    : ${BELENIOS_GROUPDIR:=demo/groups}
+    : ${BELENIOS_BINDIR:=_run/usr/bin}
+    : ${BELENIOS_LIBDIR:=_run/usr/lib}
+    : ${BELENIOS_SHAREDIR:=_run/usr/share/belenios-server}
 fi
 
 check_nonempty_var () {
@@ -19,11 +25,16 @@ check_nonempty_var () {
 }
 
 check_nonempty_var BELENIOS_CONFIG
+
+if [ -f ${BELENIOS_CONFIG}.preload ]; then
+   . ${BELENIOS_CONFIG}.preload
+fi
+
 check_nonempty_var BELENIOS_VARDIR
 check_nonempty_var BELENIOS_RUNDIR
+check_nonempty_var BELENIOS_BINDIR
 check_nonempty_var BELENIOS_LIBDIR
-check_nonempty_var BELENIOS_STATICDIR
-check_nonempty_var BELENIOS_GROUPDIR
+check_nonempty_var BELENIOS_SHAREDIR
 
 mkdir -p \
       $BELENIOS_VARDIR/etc \
@@ -31,6 +42,7 @@ mkdir -p \
       $BELENIOS_VARDIR/lib \
       $BELENIOS_VARDIR/upload \
       $BELENIOS_VARDIR/spool \
+      $BELENIOS_VARDIR/accounts \
       $BELENIOS_RUNDIR
 
 touch $BELENIOS_VARDIR/password_db.csv
@@ -39,14 +51,11 @@ sed \
     -e "s@_VARDIR_@$BELENIOS_VARDIR@g" \
     -e "s@_RUNDIR_@$BELENIOS_RUNDIR@g" \
     -e "s@_LIBDIR_@$BELENIOS_LIBDIR@g" \
-    -e "s@_STATICDIR_@$BELENIOS_STATICDIR@g" \
-    -e "s@_GROUPDIR_@$BELENIOS_GROUPDIR@g" \
+    -e "s@_SHAREDIR_@$BELENIOS_SHAREDIR@g" \
     $BELENIOS_CONFIG > $BELENIOS_VARDIR/etc/ocsigenserver.conf
 
-OCSIGENSERVER=ocsigenserver
+PATH=$BELENIOS_BINDIR:$PATH:/usr/sbin
 
-if command -v ocsigenserver.opt > /dev/null; then
-  OCSIGENSERVER=ocsigenserver.opt
-fi
+OCSIGENSERVER=belenios-server
 
 exec $OCSIGENSERVER -c $BELENIOS_VARDIR/etc/ocsigenserver.conf "$@"
