@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2021 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,6 +19,8 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+let debug x = prerr_endline x
+
 let sha256_hex x = Cryptokit.(x |>
   hash_string (Hash.sha256 ()) |>
   transform_string (Hexa.encode ())
@@ -69,7 +71,6 @@ let pbkdf2_generic toBits ~iterations ~salt x =
   pbkdf2 ~prf:MAC.hmac_sha256 ~iterations ~size:1 ~salt x |>
   transform_string (Hexa.encode ())
 
-let pbkdf2_hex = pbkdf2_generic Cryptokit.(transform_string (Hexa.decode ()))
 let pbkdf2_utf8 = pbkdf2_generic (fun x -> x)
 
 let aes_hex ~key ~data =
@@ -194,8 +195,7 @@ let decrypt ~key ~iv ~ciphertext =
 type rng = Cryptokit.Random.rng
 
 let secure_rng =
-  if Belenios_version.debug &&
-    (try Sys.getenv "BELENIOS_USE_URANDOM" with Not_found -> "") <> ""
+  if Belenios_version.debug && Sys.getenv_opt "BELENIOS_USE_URANDOM" <> None
   then Cryptokit.Random.device_rng "/dev/urandom"
   else Cryptokit.Random.secure_rng
 
@@ -204,9 +204,14 @@ let random_string = Cryptokit.Random.string
 
 module Z = struct
   include Z
+  let of_hex x = of_string_base 16 x
   let ( =% ) = equal
-  let bit_length x = Pervasives.(String.length (to_bits x) * 8)
+  let bit_length x = Stdlib.(String.length (to_bits x) * 8)
 
-  let powm = powm_sec (* Warning: no efforts have been made to be
-                         constant time in the rest of the code. *)
+  let powm x a m =
+    if Z.compare a Z.zero = 0 then Z.one else powm_sec x a m
+    (* Warning: no efforts have been made to be constant time in the
+       rest of the code. *)
+
+  let hash_to_int = Z.hash
 end
